﻿using msoui.Models;

namespace msoui.Forms;

internal sealed partial class InstallForm : Form
{
    public InstallForm(InstallModel model)
    {
        Model = model;
        InitializeComponent();
    }

    private InstallModel Model { get; set; }

    private void LogIt(string message)
    {
        logList.Items.Add(message);
        Application.DoEvents();
    }

    private void LogIt(string message, params object?[] args)
    {
        logList.Items.Add(string.Format(message, args));
        Application.DoEvents();
    }

    private void AddStatus(string message, params object?[] args)
    {
        statusLabel.Text = string.Format(message, args);
        Application.DoEvents();
    }

    private void StartProcess(string fileName)
    {
        process.StartInfo.FileName = fileName;
        process.StartInfo.Arguments = "/q /norestart";
        process.StartInfo.CreateNoWindow = false;
        process.EnableRaisingEvents = true;
        process.Exited += (_, _) => LogIt(@"[{0}] Процесс завершен с кодом {1}.", process.ExitTime, process.ExitCode);
        process.Start();
        process.WaitForExit();
    }


    private void InstallUpdates()
    {
        //Сообщение о начале установки
        LogIt("Начинаю установку...");

        //Высчитываю максимальное число программ
        int updatesCount = Model.FileList.Count;

        //Вывожу сообщение о количестве элементов
        LogIt("Выбрано {0} обновлений для установки...", updatesCount);
        AddStatus("Выбрано {0} обновлений для установки...", updatesCount);

        //Счётчик
        int count = 0;

        //Устанавливаю индикатор прогресса
        progressBar.Maximum = updatesCount;
        progressBar.Value = 0;

        //Запускаю программы установки
        foreach (string? file in Model.FileList)
        {
            //Добавляю статусное сообщение
            AddStatus("Устанавливается обновление: {0}", file);

            //Прибавляю счётчик
            count++;

            //Считаю процент выполненного
            decimal percentDone = Math.Round((decimal) count / updatesCount * 100);

            //Заношу в лог информацию об устанавливаемом файле
            LogIt(@"Устанавливается файл {0} [{1} из {2}, {3}%]...", file, count, updatesCount, percentDone);

            //Запускаю файл установки
            StartProcess($"{Model.FromDir}{file}");
            Application.DoEvents();

            //Увеличиваю прогресс
            progressBar.Value++;
            Application.DoEvents();

            //Вывожу сообщение об установке
            LogIt(@"Файл {0} установлен!", file);
        }

        //Вывожу сообщение об окончании установки
        MessageBox.Show(@"Набор обновлений успешно установлен!", @"Установка завершена", MessageBoxButtons.OK,
            MessageBoxIcon.Asterisk);

        //Закрываю окно
        DialogResult = DialogResult.OK;
    }

    private void startTimer_Tick(object sender, EventArgs e)
    {
        startTimer.Enabled = false;

        InstallUpdates();
    }

    private void InstallForm_Load(object sender, EventArgs e)
    {
        startTimer.Enabled = true;
    }
}